@@ -1,4 +1,8 @@
 expect_equal_layer <- function(x, y) {
+  # Remove constructor, which was added in ggplot2 3.4
+  x$constructor <- NULL
+  y$constructor <- NULL
+
   x$mapping <- x$mapping[sort(names(x$mapping))]
   y$mapping <- y$mapping[sort(names(y$mapping))]
   x$data <- tibble::as_tibble(x$data[, sort(colnames(x$data))])
